# Copyright (c) 2016 Boulder Real Time Technologies, Inc.
# All rights reserved

# This software module is wholly owned by Boulder Real Time
# Technologies, Inc. Any use of this software module without
# express written permission from Boulder Real Time Technologies,
# Inc. is prohibited.

import _Pkt

if __name__ == '__main__':
    import unittest
    import warnings

    import _stock

    class TestRawPkt(unittest.TestCase):

        def test_procedure_is_stash(self):

            self.assertFalse(_Pkt._is_stash("AZ_PFO_BHZ_00/BBA/BS", b"Not a valid stash packet buffer"))
            self.assertTrue(_Pkt._is_stash("AZ_PFO_BHZ_00/BBA/BS", b"\xff\x01\xbf\x80\x00"))

        def test_procedure_unstuffPkt(self):

            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")
                _type, pkt = _Pkt._unstuffPkt("AZ_PFO_BHZ_00/BBA/BS", 1329775358, b'')
                self.assertEqual(len(w), 1)
                self.assertEqual(w[-1].category, RuntimeWarning)
                self.assertTrue("Failure to unstuff packet" in str(w[-1].message))
                self.assertEqual(_type, -1)

        def test_procedure_unstuffPkt_type(self):
            _type, pkt = _Pkt._unstuffPkt("AZ_PFO_BHZ_00/BBA/BS",
                                          1329775358,
                                          b'\xff\x01\xbf\x80\x00\x00')
            self.assertEqual(_type, _Pkt.Pkt_stash)

        def test_procedure_stuffPkt(self):

            pkt = _Pkt._newPkt()

            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")
                _type, packet, srcname, time = _Pkt._stuffPkt(pkt)
                self.assertEqual(len(w), 1)
                self.assertEqual(w[-1].category, RuntimeWarning)
                self.assertTrue("Cannot stuff packet with invalid packet suffix" in str(w[-1].message))

            _Pkt._freePkt(pkt)

        def test_procedure_stuffStashPkt(self):

            stash = "Stashed info for a particular srcname"

            _type, packet = _Pkt._stuffStashPkt(stash)

            self.assertEqual(_type, _Pkt.Pkt_stash)

        def test_procedure_newPkt(self):

            pkt = _Pkt._newPkt()

            self.assertIsInstance(pkt, str)

            _Pkt._freePkt(pkt)

        def test_procedure_clrPkt(self):

            pkt = _Pkt._newPkt()

            _Pkt._clrPkt(pkt)

            _Pkt._freePkt(pkt)

            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")
                _Pkt._clrPkt("DNEDNEDNE")
                self.assertEqual(len(w), 1)
                self.assertEqual(w[-1].category, RuntimeWarning)
                self.assertTrue("parse_to_addr: could not find address handle 'DNEDNEDNE'" in str(w[-1].message))

        def test_procedure_freePkt(self):

            pkt = _Pkt._newPkt()

            _Pkt._freePkt(pkt)

        def test_procedure_newPktChannel(self):

            pktchan = _Pkt._newPktChannel()

            self.assertIsInstance(pktchan, str)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_clrPktChannel(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._clrPktChannel(pktchan)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_freePktChannel(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._freePktChannel(pktchan)

        def test_procedure_Pkt_type_get(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_pkttype_set(pkt, "BBA")

            _type = _Pkt._Pkt_type_get(pkt)

            self.assertEqual(_type, _Pkt.Pkt_wf)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_pkttype_get(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_pkttype_set(pkt, "BBA")

            _type, name, suffix, hdrcode, bodycode, desc = _Pkt._Pkt_pkttype_get(pkt)

            self.assertEqual(_type, _Pkt.Pkt_wf)
            self.assertEqual(name, "BBA")
            self.assertEqual(suffix, "BBA")
            self.assertEqual(hdrcode, 3002)
            self.assertEqual(bodycode, 4294957739)
            self.assertEqual(desc, "Broadband array data stream 5/18/01")

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_time_get(self):

            pkt = _Pkt._newPkt()

            time = _Pkt._Pkt_time_get(pkt)

            self.assertEqual(time, 0)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_time_set(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_time_set(pkt, 1329775358)

            time = _Pkt._Pkt_time_get(pkt)

            self.assertEqual(time, 1329775358)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_nchannels_get(self):

            pkt = _Pkt._newPkt()

            nchannels = _Pkt._Pkt_nchannels_get(pkt)

            self.assertEqual(nchannels, 0)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_nchannels_set(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_nchannels_set(pkt, 25)

            nchannels = _Pkt._Pkt_nchannels_get(pkt)

            self.assertEqual(nchannels, 25)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_srcnameparts_get(self):

            pkt = _Pkt._newPkt()

            net, sta, chan, loc, suffix, subcode = _Pkt._Pkt_srcnameparts_get(pkt)

            self.assertEqual(net, "")
            self.assertEqual(sta, "")
            self.assertEqual(chan, "")
            self.assertEqual(loc, "")
            self.assertEqual(suffix, "")
            self.assertEqual(subcode, "")

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_srcnameparts_set(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_srcnameparts_set(pkt, "AZ", "PFO", "BHZ", "00", "BBA", "BS")

            net, sta, chan, loc, suffix, subcode = _Pkt._Pkt_srcnameparts_get(pkt)

            self.assertEqual(net, "AZ")
            self.assertEqual(sta, "PFO")
            self.assertEqual(chan, "BHZ")
            self.assertEqual(loc, "00")
            self.assertEqual(suffix, "BBA")
            self.assertEqual(subcode, "BS")

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_channels_get(self):

            pkt = _Pkt._newPkt()

            channels = _Pkt._Pkt_channels_get(pkt)

            self.assertIsInstance(channels, tuple)
            self.assertEqual(len(channels), 0)

        def test_procedure_Pkt_channels_set(self):

            pkt = _Pkt._newPkt()

            channels = list()

            channels.append(_Pkt._newPktChannel())
            channels.append(_Pkt._newPktChannel())
            channels.append(_Pkt._newPktChannel())

            _Pkt._Pkt_channels_set(pkt, channels)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_channels_append(self):

            pkt = _Pkt._newPkt()

            channels = list()

            channels.append(_Pkt._newPktChannel())
            channels.append(_Pkt._newPktChannel())
            channels.append(_Pkt._newPktChannel())

            _Pkt._Pkt_channels_set(pkt, channels)

            morechannels = list()

            morechannels.append(_Pkt._newPktChannel())
            morechannels.append(_Pkt._newPktChannel())

            _Pkt._Pkt_channels_append(pkt, morechannels)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_channels_chan_get(self):

            pkt = _Pkt._newPkt()

            channels = list()

            channels.append(_Pkt._newPktChannel())
            channels.append(_Pkt._newPktChannel())
            channels.append(_Pkt._newPktChannel())

            _Pkt._Pkt_channels_set(pkt, channels)

            pktchan = _Pkt._Pkt_channels_chan_get(pkt, 1)

            self.assertTrue(pktchan == channels[1])

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_pf(self):

            pkt = _Pkt._newPkt()

            rc, pfptr = _stock._pfread("rtexec")

            _Pkt._Pkt_pfptr_set(pkt, pfptr)

            _stock._pffree(pfptr)

            pfptr = _Pkt._Pkt_pfptr_get(pkt)

            self.assertEqual(pfptr[0:2], "Pf")

            _stock._pffree(pfptr)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_db(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_db_set(pkt, [0, -501, -501, -501])

            db = _Pkt._Pkt_db_get(pkt)

            self.assertEqual(db, [0, -501, -501, -501])

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_string(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_string_set(pkt, "teststring")

            string = _Pkt._Pkt_string_get(pkt)

            self.assertEqual(string, "teststring")

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_dfile(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_dfile_set(pkt, "testdfile")

            dfile = _Pkt._Pkt_dfile_get(pkt)

            self.assertEqual(dfile, "testdfile")

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_version(self):

            pkt = _Pkt._newPkt()

            _Pkt._Pkt_version_set(pkt, 5)

            version = _Pkt._Pkt_version_get(pkt)

            self.assertEqual(version, 5)

            _Pkt._freePkt(pkt)

        def test_procedure_Pkt_hook(self):

            pkt = _Pkt._newPkt()

            hook = _Pkt._Pkt_hook_get(pkt)

            self.assertEqual(hook, None)

            _Pkt._freePkt(pkt)

        def test_procedure_PktChannel_data(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_data_set(pktchan, [1, 2, 3, 4, 5])

            data = _Pkt._PktChannel_data_get(pktchan)

            self.assertEqual(data, (1, 2, 3, 4, 5))

            self.assertFalse(_Pkt._PktChannel_isfloat_get(pktchan))

            _Pkt._PktChannel_data_set(pktchan, (1.5, 2.5, 3.5, 4.5, 5.5))

            data = _Pkt._PktChannel_data_get(pktchan)

            self.assertEqual(data, (1.5, 2.5, 3.5, 4.5, 5.5))

            self.assertTrue(_Pkt._PktChannel_isfloat_get(pktchan))

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_time(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_time_set(pktchan, 1330458027)

            time = _Pkt._PktChannel_time_get(pktchan)

            self.assertEqual(time, 1330458027)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_samprate(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_samprate_set(pktchan, 100)

            samprate = _Pkt._PktChannel_samprate_get(pktchan)

            self.assertEqual(samprate, 100)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_calib(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_calib_set(pktchan, 1)

            calib = _Pkt._PktChannel_calib_get(pktchan)

            self.assertEqual(calib, 1)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_calper(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_calper_set(pktchan, 1)

            calper = _Pkt._PktChannel_calper_get(pktchan)

            self.assertEqual(calper, 1)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_duser1(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_duser1_set(pktchan, 58.5)

            duser1 = _Pkt._PktChannel_duser1_get(pktchan)

            self.assertEqual(duser1, 58.5)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_duser2(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_duser2_set(pktchan, 63.4)

            duser2 = _Pkt._PktChannel_duser2_get(pktchan)

            self.assertEqual(duser2, 63.4)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_nsamp(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_nsamp_set(pktchan, 2000)

            nsamp = _Pkt._PktChannel_nsamp_get(pktchan)

            self.assertEqual(nsamp, 2000)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_iuser1(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_iuser1_set(pktchan, 456)

            iuser1 = _Pkt._PktChannel_iuser1_get(pktchan)

            self.assertEqual(iuser1, 456)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_iuser2(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_iuser2_set(pktchan, 572)

            iuser2 = _Pkt._PktChannel_iuser2_get(pktchan)

            self.assertEqual(iuser2, 572)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_iuser3(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_iuser3_set(pktchan, 1346)

            iuser3 = _Pkt._PktChannel_iuser3_get(pktchan)

            self.assertEqual(iuser3, 1346)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_net(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_net_set(pktchan, "AZ")

            net = _Pkt._PktChannel_net_get(pktchan)

            self.assertEqual(net, "AZ")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_sta(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_sta_set(pktchan, "PFO")

            sta = _Pkt._PktChannel_sta_get(pktchan)

            self.assertEqual(sta, "PFO")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_chan(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_chan_set(pktchan, "BHZ")

            chan = _Pkt._PktChannel_chan_get(pktchan)

            self.assertEqual(chan, "BHZ")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_loc(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_loc_set(pktchan, "00")

            loc = _Pkt._PktChannel_loc_get(pktchan)

            self.assertEqual(loc, "00")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_segtype(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_segtype_set(pktchan, "V")

            segtype = _Pkt._PktChannel_segtype_get(pktchan)

            self.assertEqual(segtype, "V")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_cuser1(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_cuser1_set(pktchan, "A String 1")

            cuser1 = _Pkt._PktChannel_cuser1_get(pktchan)

            self.assertEqual(cuser1, "A String 1")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_cuser2(self):

            pktchan = _Pkt._newPktChannel()

            _Pkt._PktChannel_cuser2_set(pktchan, "A String 2")

            cuser2 = _Pkt._PktChannel_cuser2_get(pktchan)

            self.assertEqual(cuser2, "A String 2")

            _Pkt._freePktChannel(pktchan)

        def test_procedure_PktChannel_hook(self):

            pktchan = _Pkt._newPktChannel()

            hook = _Pkt._PktChannel_hook_get(pktchan)

            self.assertEqual(hook, None)

            _Pkt._freePktChannel(pktchan)

        def test_procedure_split_srcname(self):

            net, sta, chan, loc, suffix, subcode = _Pkt._split_srcname("AZ_PFO_BHZ_00/BBA/BS")

            self.assertEqual(net, "AZ")
            self.assertEqual(sta, "PFO")
            self.assertEqual(chan, "BHZ")
            self.assertEqual(loc, "00")
            self.assertEqual(suffix, "BBA")
            self.assertEqual(subcode, "BS")

        def test_procedure_join_srcname(self):

            srcname = _Pkt._join_srcname("AZ", "PFO", "BHZ", "00", "BBA", "BS")

            self.assertEqual(srcname, "AZ_PFO_BHZ_00/BBA/BS")

    unittest.main()
